(function () {
	if ($("form#dashboard_form").length == 0) {
		return; // tab gone, abort
	}
	var dashboardInterval = 10000;
	var resources = [], activeCalls = [], activeChannels = [];
	var network_state = null;

	/* Plot Settings */
	var plotSeries = {
		lines:{
			show: true,
			fill: .3,
			lineWidth: 1,
			fillColor:{
				colors: [ { opacity: 0.2 },{ opacity: 0.9 } ]
			}
		},
		points: {
			show: false,
			lineWidth: 1.5,
			radius: 2,
			symbol: "circle",
			fill: true,
			fillColor: "#ffffff"
		},
		shadowSize: 0,
		curvedLines: {
			active: true,
			apply: true,
			fit: true,
			fitPointDist: 0.000001
		}
	};

	var plotGrid = {
		hoverable: true,
		clickable: true,
		borderColor: "#fff",
		borderWidth: 0,
		labelMargin: 10,
		backgroundColor: "#fff"
	};

	var plotXAxis = {
		mode: "time",
		color: 'rgba(0,0,0,0)',
		tickSize: [60, "second"],
		tickFormatter: function (v, axis) {
			var date = new Date(v);
			var hours = date.getHours() < 10 ? "0" + date.getHours() : date.getHours();
			var minutes = date.getMinutes() < 10 ? "0" + date.getMinutes() : date.getMinutes();
			var seconds = date.getSeconds() < 10 ? "0" + date.getSeconds() : date.getSeconds();
			return hours + ":" + minutes;
		},
		axisLabel: "Time"
	};

	initAsteriskData();
	hardDiskPie();
	register_timer("dashboard", "update", getDashboardInfo, dashboardInterval, true);
	$("a[href='#tab_dashboard'] i.reload-tab").remove();

	function getDashboardInfo(){
		var tab = $('ul#ombutel-main-tabs > li').find('a[aria-controls="tab_dashboard"]');
		//Request to Server
		$.ajax({
			dataType: 'json',
			data: { class: 'dashboard', method: 'getDashboardData', 'network_state': network_state },
			success: function (Response) {
				var now = new Date().getTime();
				updateUptime(
					Response.current_time,
					Response.asterisk.asterisk_uptime,
					Response.server_uptime
				);
				if($('#cpu-placeholder').length > 0){
					updateCPUChart(now, Response.cpu_info);
				}

				if($('#asterisk-call-placeholder').length > 0){
					updateAsteriskChart(now, Response.asterisk);
				}
				updateServicesStatus(Response.services_status);
				network_state = Response.networks.state;
				delete Response.networks.state;
				updateNetworkUsage(Response.networks);
			}
		});
	}

	function updateUptime(current_time, asteriskUptime, serverUptime) {
		$('span.span-current_time').text(current_time);
		$('span.span-asterisk_uptime').text(asteriskUptime);
		$('span.span-server_uptime').text(serverUptime);
	}

	function hardDiskPie(){
		if ($("#hdd-placeholder").length > 0) {
			$.plot("#hdd-placeholder", HardDiskDataSet, {
				series: {
					 pie: {
						show: true,
						combine: {
							color: "#999",
							threshold: 0.05
						}
					}
				},
				legend: {
					show: false
				},
				grid: {
					/*hoverable: true*/
				}
			});
		}
	}

	function updateServicesStatus(data) {
		var icons = {"active": "fa-check-circle", "inactive": "fa-times-circle", "unknown": "fa-exclamation-circle"};
		$.each(data, function (i,value) {
			var span = $('.span-' + i);
			var icon = $('.icon-' + i);
			var serviceClass = 'dash-service-' + value;

			icon.removeClass('fa-check-circle fa-times-circle fa-exclamation-circle dash-service-active dash-service-stopped dash-service-unknown');
			icon.addClass(icons[value] + " " + serviceClass);
			span.text(value);
		});
	}

	function updateNetworkUsage(data) {
		var networkUL = $("ul#dash-network-items");
		networkUL.html('');
		$.each(data, function(interface, current) {
			if (interface != 'time') {
				networkUL.append(networkTemplate(interface, current));
			}
		});
	}

	function networkTemplate(interface, current) {
		return '<li> ' +
			'<div class="icon-section"> <i class="fa fa-cloud fa-fw"></i> </div>' +
			'<div class="content-section"> ' +
			'<span class="dash-label"> '+ interface +' </span> ' +
			'<span class="desc"> '+ current +' </span> ' +
			'</div>' +
			' </li>';
	}

	function initAsteriskData() {
		var now = new Date().getTime();
		for (var i = 20; i > 0; --i) {
			temp = [now - (dashboardInterval * i), 0];
			activeCalls.push(temp);
			activeChannels.push(temp);
		}
	}

	function initChartsData(cpuInfo){
		var now = new Date().getTime();
		for(resource in cpuInfo) {
			if (!(resource in resources)) {
				resources[resource] = [];
				for (var i = 20; i > 0; --i) {
					temp = [now - (dashboardInterval * i), 0];
					resources[resource].push(temp);
				}
			}
		}
	}

	function updateCPUChart(now, cpuInfo){
		var temp;
		var dataSet = [];

		if (resources.length == 0) {
			initChartsData(cpuInfo);
		}

		for(resource in cpuInfo) {
			temp = [now, cpuInfo[resource]];
			resources[resource].shift();
			resources[resource].push(temp);
			dataSet.push({ label: eval("ombu.i18n.dashboard." + resource), data: resources[resource], color: get_color(resource) });
		}

		$.plot($('#cpu-placeholder'),dataSet,{
			series: plotSeries,
			legend: {
				position: "nw"
			},
			grid: plotGrid,
			tooltip: true,
			xaxis: plotXAxis,
			yaxis:{
				min: 0,
				max: 100,
				tickSize: 5,
				tickFormatter: function (v, axis) {
					if (v % 10 == 0) {
						return v + "%";
					} else {
						return "";
					}
				},
				color: 'rgba(0,0,0,0)',
				axisLabel: ombu.i18n.dashboard.percentage,
				axisLabelUseCanvas: true,
				axisLabelFontSizePixels: 12,
				axisLabelFontFamily: 'Verdana, Arial',
				axisLabelPadding: 6
			}
		});

		$('#cpu-placeholder').bind("plothover", function (event, pos, item){
			if(item){
				var x = item.datapoint[0],
					y = item.datapoint[1].toFixed(2);

				$("#flotTip").remove();
				showPlotTooltip(item.pageX, item.pageY, item.series.label +': ' + y + '%');
			}else{
				$("#flotTip").remove();
			}
		});
	}

	function get_color(resource) {
		switch (resource) {
			case 'cpu' : { return "#3498db"; }
			case 'memory': { return "#e67e22"; }
			case 'swap': { return "#e74c3c"; }
			case 'loadavg': { return "#8e44ad"; }
		}
	}

	function updateAsteriskChart(now, asteriskInfo){
		activeCalls.shift();
		activeChannels.shift();
		var temp;
		temp = [now, asteriskInfo.active_calls];
		activeCalls.push(temp);

		temp = [now, asteriskInfo.active_channels];
		activeChannels.push(temp);

		var dataSet = [
			{ label: ombu.i18n.dashboard.active_calls, data: activeCalls, color: "#00BCD4" },
			{ label: ombu.i18n.dashboard.active_channels, data: activeChannels, color: "#795548" },
		];

		$.plot($('#asterisk-call-placeholder'),dataSet,{
			series: plotSeries,
			legend: {
				position: "nw"
			},
			grid: plotGrid,
			tooltip: true,
			xaxis: plotXAxis,
			yaxis:{
				min: 0,
				tickDecimals:0,
				color: 'rgba(0,0,0,0)',
				axisLabel: ombu.i18n.dashboard.calls,
				axisLabelUseCanvas: true,
				axisLabelFontSizePixels: 12,
				axisLabelFontFamily: 'Verdana, Arial',
				axisLabelPadding: 6
			}
		});

		$('#asterisk-call-placeholder').bind("plothover", function (event, pos, item){
			if(item){
				var x = item.datapoint[0],
					y = item.datapoint[1].toFixed(0);

				$("#flotTip").remove();
				showPlotTooltip(item.pageX, item.pageY, item.series.label +': ' + y);
			}else{
				$("#flotTip").remove();
			}
		});
	}

	function showPlotTooltip(x, y, contents) {
		$('<div id="flotTip">' + contents + '</div>').css( {
			position: "absolute",
			top: y + 5,
			left: x + 5
		}).appendTo("body").fadeIn(200);
	}
})();
